package com.ldjam.characters;

import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;

import com.badlogic.gdx.graphics.Texture;
import com.badlogic.gdx.graphics.g2d.Animation.PlayMode;
import com.badlogic.gdx.graphics.g2d.TextureRegion;
import com.ldjam.game.GdxUtils;
import com.ldjam.game.Item;
import com.ldjam.game.OneRoom;
import com.ldjam.game.Shoot;
import com.ldjam.game.Item.ITEM_TYPE;

public class BlackSoldier extends Peasant{

	protected List<Shoot> shoots;
	protected float shootSpeed = 3.4f;
	protected TextureRegion[][] shootFrames;
	
	public BlackSoldier(float x, float y) {
		super(x, y);
		speed = 1.2f;
		Texture texture = GdxUtils.TEXTURE_BLACK_SOLDIER;
		
		peasentTR = TextureRegion.split(texture, 32, 32);
		walkAnimations[DIRECTION_DOWN] = GdxUtils.loadAnimation(peasentTR, 3, DIRECTION_DOWN, 0.12f);
		walkAnimations[DIRECTION_UP] = GdxUtils.loadAnimation(peasentTR, 3, DIRECTION_UP, 0.12f);
		walkAnimations[DIRECTION_LEFT] = GdxUtils.loadAnimation(peasentTR, 3, DIRECTION_LEFT, 0.12f);
		walkAnimations[DIRECTION_RIGHT] = GdxUtils.loadAnimation(peasentTR, 3, DIRECTION_RIGHT, 0.12f);
		for(int i=0;i<walkAnimations.length;i++){
			walkAnimations[i].setPlayMode(PlayMode.LOOP);
		}
		shoots = new ArrayList<Shoot>();
		nearPlayerDistance = 256;
		this.maxLife=4;
		this.life = maxLife;
		shootFrames = TextureRegion.split(GdxUtils.TEXTURE_SWORD_SLASH, 32, 32);
		maxCooldownAttack = 45;
	}
	
	@Override
	public void loot(OneRoom game){

		double rand = Math.random();
		if(rand>0.1){
			game.getItems().add(new Item(x, y, ITEM_TYPE.HEART, true));
		}
		game.getItems().add(new Item(x, y, ITEM_TYPE.POTION, true));
		game.getItems().add(new Item(x, y, ITEM_TYPE.XP_V, true));
		game.getItems().add(new Item(x, y, ITEM_TYPE.XP_V, true));
		game.getItems().add(new Item(x, y, ITEM_TYPE.XP_I, true));
		game.getItems().add(new Item(x, y, ITEM_TYPE.XP_I, true));
		destroy = true;
	}
	
	@Override
	public void update(OneRoom game){
		super.update(game);
		
		Iterator<Shoot> it = shoots.iterator();
		while(it.hasNext()){
			Shoot s = it.next();
			if(s.isDestroy()){
				it.remove();
			}else{
				s.update(game);
				s.touchPlayer(game);
			}
		}
	}
	
	@Override
	public void render(OneRoom game){
		super.render(game);
		
		for (Shoot shoot : shoots) {
			shoot.render(game);
		}
	}

	@Override
	public void shoot(OneRoom game){
		
		if(life>maxLife/2){
			Shoot newShoot = null;

			if(direction == DIRECTION_DOWN){
				newShoot = new Shoot(x,y,0,-shootSpeed,shootFrames[direction][0]);
			}else if(direction == DIRECTION_UP){
				
				newShoot = new Shoot(x,y,0,shootSpeed,shootFrames[direction][0]);
			}else if(direction == DIRECTION_LEFT){
				newShoot = new Shoot(x,y,-shootSpeed,0,shootFrames[direction][0]);
			}else if(direction == DIRECTION_RIGHT){
				newShoot = new Shoot(x,y,shootSpeed,0,shootFrames[direction][0]);
			}
			newShoot.setSize(32);
			shoots.add(newShoot);
		}else{
			Shoot s1 = new Shoot(x,y,0,-shootSpeed,shootFrames[DIRECTION_DOWN][0]);
			Shoot s2 = new Shoot(x,y,0,shootSpeed,shootFrames[DIRECTION_UP][0]);
			Shoot s3 = new Shoot(x,y,-shootSpeed,0,shootFrames[DIRECTION_LEFT][0]);
			Shoot s4 = new Shoot(x,y,shootSpeed,0,shootFrames[DIRECTION_RIGHT][0]);
			s1.setSize(32);
			s2.setSize(32);
			s3.setSize(32);
			s4.setSize(32);
			shoots.add(s1);
			shoots.add(s2);
			shoots.add(s3);
			shoots.add(s4);
		}
	}
	
}
