package com.ldjam.characters;

import com.badlogic.gdx.graphics.g2d.Animation;
import com.badlogic.gdx.graphics.g2d.TextureRegion;
import com.badlogic.gdx.graphics.g2d.Animation.PlayMode;
import com.ldjam.game.Entity;
import com.ldjam.game.GameAnimation;
import com.ldjam.game.GdxUtils;
import com.ldjam.game.OneRoom;
import com.ldjam.game.SimpleAnimation;

public class BodyPart extends Entity implements GameAnimation{

	protected TextureRegion currentFrame;
	protected float dx, dy;
	protected int duration;
	protected int duration2;
	protected static Animation[] bloodAnimations;
	
	public BodyPart(float x, float y, TextureRegion currentFrame, boolean move) {
		super(x, y);
		if(move){
			this.dx = (float) ((Math.random()*4)-2);
			this.dy = (float) ((Math.random()*4)-2);
			
		}else{
			this.dx = 0;
			this.dy = 0;
		}
		duration =(int)(Math.random()*32+16);
		duration2 =(int)(Math.random()*1024+512);
		if(!move){
			duration2 = 4*duration2;
		}
		this.currentFrame = currentFrame;
		
		if(bloodAnimations == null){
			TextureRegion[][] bloodTR = TextureRegion.split(GdxUtils.TEXTURE_BLOOD, 16, 16);
			bloodAnimations = new Animation[8];
			for(int i=0;i<8;i++){
				bloodAnimations[i] = GdxUtils.loadAnimation(bloodTR, 6, i, 0.08f);
				bloodAnimations[i].setPlayMode(PlayMode.NORMAL);
			}
		}
	}
	
	public void update(OneRoom game){
		if(duration>0){
			duration--;
			if(!game.getRoom().isBlockedRestricted(x+dx, y+dy)){
				x+=dx;
				y+=dy;
				if(Math.random()<0.35){
					game.addAnimation(new SimpleAnimation(x-8,y-8,bloodAnimations[(int) (Math.random()*8)]));
				}
			}
		}else if(duration2>0){
			duration2--;
		}
	}
	
	public void render(OneRoom game){
		if(duration2<15){
			game.getBatch().setColor(1, 1, 1, 0.75f);
		}else if(duration2<10){
			game.getBatch().setColor(1, 1, 1, 0.5f);
		}else if(duration2<5){
			game.getBatch().setColor(1, 1, 1, 0.25f);
		}
		game.getBatch().draw(currentFrame, x, y);
		game.getBatch().setColor(1, 1, 1,1f);

	}
	
	public boolean isFinish(){
		return duration<=0 && duration2<=0;
	}

}
