package com.ldjam.characters;

import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;

import com.badlogic.gdx.graphics.Texture;
import com.badlogic.gdx.graphics.g2d.Animation.PlayMode;
import com.badlogic.gdx.graphics.g2d.TextureRegion;
import com.ldjam.game.GdxUtils;
import com.ldjam.game.Item;
import com.ldjam.game.OneRoom;
import com.ldjam.game.Shoot;
import com.ldjam.game.Item.ITEM_TYPE;

public class GoodWizard extends Peasant{

	protected List<Shoot> shoots;
	protected float shootSpeed = 3.6f;
	protected TextureRegion shootFrame;
	
	public GoodWizard(float x, float y) {
		super(x, y);
		speed = 1.4f;
		Texture texture = GdxUtils.TEXTURE_GOOD_WIZARD;
		
		peasentTR = TextureRegion.split(texture, 32, 32);
		walkAnimations[DIRECTION_DOWN] = GdxUtils.loadAnimation(peasentTR, 3, DIRECTION_DOWN, 0.12f);
		walkAnimations[DIRECTION_UP] = GdxUtils.loadAnimation(peasentTR, 3, DIRECTION_UP, 0.12f);
		walkAnimations[DIRECTION_LEFT] = GdxUtils.loadAnimation(peasentTR, 3, DIRECTION_LEFT, 0.12f);
		walkAnimations[DIRECTION_RIGHT] = GdxUtils.loadAnimation(peasentTR, 3, DIRECTION_RIGHT, 0.12f);
		for(int i=0;i<walkAnimations.length;i++){
			walkAnimations[i].setPlayMode(PlayMode.LOOP);
		}
		shoots = new ArrayList<Shoot>();
		nearPlayerDistance = 256;
		this.maxLife=4;
		this.life = maxLife;
		shootFrame = new TextureRegion(GdxUtils.TEXTURE_WIZARD_STARSHOOT);
	}
	
	@Override
	public void loot(OneRoom game){

		double rand = Math.random();
		if(rand>0.1){
			game.getItems().add(new Item(x, y, ITEM_TYPE.HEART, true));
		}
		game.getItems().add(new Item(x, y, ITEM_TYPE.POTION, true));
		game.getItems().add(new Item(x, y, ITEM_TYPE.XP_V, true));
		game.getItems().add(new Item(x, y, ITEM_TYPE.XP_V, true));
		game.getItems().add(new Item(x, y, ITEM_TYPE.XP_V, true));
		game.getItems().add(new Item(x, y, ITEM_TYPE.XP_I, true));
		destroy = true;
	}
	
	@Override
	public void update(OneRoom game){
		super.update(game);
		
		Iterator<Shoot> it = shoots.iterator();
		while(it.hasNext()){
			Shoot s = it.next();
			if(s.isDestroy()){
				it.remove();
			}else{
				s.update(game);
				s.touchPlayer(game);
			}
		}
	}
	
	@Override
	public void render(OneRoom game){
		super.render(game);
		
		for (Shoot shoot : shoots) {
			shoot.render(game);
		}
	}

	@Override
	protected void updateAttack(OneRoom game){
		if(cooldownAttack<=0 && !onAttack && nearPlayer(game,nearPlayerDistance)){
			stateTimeWeapon = 0f;
			cooldownAttack = 40;
			shoot(game);
			GdxUtils.SOUND_PEASANT_ATTACK.play();
		}else if(!onAttack && cooldownAttack>0){
			cooldownAttack--;
		}
	}
	
	protected void initUniqMessage(OneRoom game){
		Double random = Math.random();
		if(life==1){
			if(random<0.25){
				message.initMessage(x, y+24, "Stupid magic.");
			}else if(random<0.5){
				message.initMessage(x, y+24, "Is that all you can do?");
			}else{
				message.initMessage(x, y+24, msgs[(int)(Math.random()*msgs.length-1)]);
			}
		}else{
			if(random<0.5){
				message.initMessage(x, y+24, "I'm a better wizard than you !");
			}else{
				message.initMessage(x, y+24, "You don't deserve to be a wizard !");
			}
		}
	}
	
	@Override
	public void shoot(OneRoom game){
		Shoot s1,s2,s3,s4;

		if(life>1){
			if(Math.random()>0.5){
				s1 = new Shoot(x,y,0,-shootSpeed,shootFrame);
				s2 = new Shoot(x,y,0,shootSpeed,shootFrame);
				s3 = new Shoot(x,y,-shootSpeed,0,shootFrame);
				s4 = new Shoot(x,y,shootSpeed,0,shootFrame);
			}else{
				s1 = new Shoot(x,y,-shootSpeed*0.66f,-shootSpeed*0.66f,shootFrame);
				s2 = new Shoot(x,y,shootSpeed*0.66f,shootSpeed*0.66f,shootFrame);
				s3 = new Shoot(x,y,-shootSpeed*0.66f,shootSpeed*0.66f,shootFrame);
				s4 = new Shoot(x,y,shootSpeed*0.66f,-shootSpeed*0.66f,shootFrame);
			}
			s1.setSize(16);
			s2.setSize(16);
			s3.setSize(16);
			s4.setSize(16);
			shoots.add(s1);
			shoots.add(s2);
			shoots.add(s3);
			shoots.add(s4);
		}else{
			Shoot s5,s6,s7,s8;
			s1 = new Shoot(x,y,0,-shootSpeed,shootFrame);
			s2 = new Shoot(x,y,0,shootSpeed,shootFrame);
			s3 = new Shoot(x,y,-shootSpeed,0,shootFrame);
			s4 = new Shoot(x,y,shootSpeed,0,shootFrame);
			s5 = new Shoot(x,y,-shootSpeed*0.66f,-shootSpeed*0.66f,shootFrame);
			s6 = new Shoot(x,y,shootSpeed*0.66f,shootSpeed*0.66f,shootFrame);
			s7 = new Shoot(x,y,-shootSpeed*0.66f,shootSpeed*0.66f,shootFrame);
			s8 = new Shoot(x,y,shootSpeed*0.66f,-shootSpeed*0.66f,shootFrame);
			s1.setSize(16);
			s2.setSize(16);
			s3.setSize(16);
			s4.setSize(16);
			s5.setSize(16);
			s6.setSize(16);
			s7.setSize(16);
			s8.setSize(16);
			shoots.add(s1);
			shoots.add(s2);
			shoots.add(s3);
			shoots.add(s4);
			shoots.add(s5);
			shoots.add(s6);
			shoots.add(s7);
			shoots.add(s8);
		}
		
	}
	
}
