package com.ldjam.characters;


import com.badlogic.gdx.Gdx;
import com.badlogic.gdx.graphics.Texture;
import com.badlogic.gdx.graphics.g2d.Animation;
import com.badlogic.gdx.graphics.g2d.TextureRegion;
import com.badlogic.gdx.graphics.g2d.Animation.PlayMode;
import com.ldjam.game.GdxUtils;
import com.ldjam.game.Item;
import com.ldjam.game.OneRoom;
import com.ldjam.game.Item.ITEM_TYPE;


public class Peasant extends Personnage{

	protected Animation[] walkAnimations = new Animation[4];
	protected Animation swordHitAnimation;
	protected float stateTimeWeapon = 0f;
	protected boolean onAttack = false;
	protected int cooldownAttack = 0, cooldownTarget = 0;
	protected TextureRegion[][] peasentTR;
	protected int nearPlayerDistance = 48;
	protected int delay=0;
	protected int maxCooldownAttack = 25;
	
	protected String[] msgs = 
		{
				"You will die evil wizard !",
				"DIE DIE DIE !",
				"You killed all my family BASTARD !",
				"Stop running !",
				"I will kill you.",
				"Can you just die ?",
				"Don't be afraid little wizard.",
				"Stop doing that !"
		};


	public Peasant(float x, float y) {
		super(x, y);
		speed = 1.75f;
		Texture texture = GdxUtils.TEXTURE_PEASANT;
		int rand = (int)(Math.random()*3);
		if(rand==1){
			texture = GdxUtils.TEXTURE_PEASANT2;
		}else if(rand==2){
			texture = GdxUtils.TEXTURE_PEASANT3;
		}

		peasentTR = TextureRegion.split(texture, 32, 32);
		walkAnimations[DIRECTION_DOWN] = GdxUtils.loadAnimation(peasentTR, 3, DIRECTION_DOWN, 0.12f);
		walkAnimations[DIRECTION_UP] = GdxUtils.loadAnimation(peasentTR, 3, DIRECTION_UP, 0.12f);
		walkAnimations[DIRECTION_LEFT] = GdxUtils.loadAnimation(peasentTR, 3, DIRECTION_LEFT, 0.12f);
		walkAnimations[DIRECTION_RIGHT] = GdxUtils.loadAnimation(peasentTR, 3, DIRECTION_RIGHT, 0.12f);
		for(int i=0;i<walkAnimations.length;i++){
			walkAnimations[i].setPlayMode(PlayMode.LOOP);
		}

		TextureRegion[][] regionsSword = TextureRegion.split(GdxUtils.TEXTURE_SWORD_HIT, 64, 64);
		swordHitAnimation = GdxUtils.loadAnimation(regionsSword, 4, 0, 0.08f);
		swordHitAnimation.setPlayMode(PlayMode.NORMAL);

		this.maxLife=2;
		this.life = maxLife;
		this.delay = 5;
	}

	/**
	 * Met a jour le paysan
	 */
	public void update(OneRoom game){
		if(delay>0 || game.getPlayer().getLife()<=0){
			delay--;
		}else{
			if(life >0){
				super.update(game);


				if(cooldownTarget<=0){
					if(Math.random()<0.1){
						setTarget((int)(Math.random()*500+70), (int)(Math.random()*250+64));

					}else{
						setTarget(game.getPlayer().getX(), game.getPlayer().getY());
					}
					cooldownTarget = (int)(Math.random()*96);
				}else{
					cooldownTarget--;
				}

				// Met a jour la position
				updateMove(game);

				// Met a jour l'attaque
				updateAttack(game);
				
				if(!message.isActive() && Math.random()<0.001){
					writeMessage(game);
				}
			}
			message.update(game);
			
		}
	}

	public void render(OneRoom game){
		if(delay>0){
			delay--;
		}else{
			if(life > 0){
				if(cooldownTouch>0){
					game.getBatch().setColor(1f, colorAfterHit, colorAfterHit, 1f);
				}
				game.getBatch().draw(walkAnimations[direction].getKeyFrame(stateTime), x-16, y-16);
				if(onAttack){
					game.getBatch().draw(swordHitAnimation.getKeyFrame(stateTimeWeapon), x-32, y-32);
				}
				if(cooldownTouch>0){
					game.getBatch().setColor(1f, 1f, 1f, 1f);
				}
				if(move){
					stateTime += Gdx.graphics.getDeltaTime(); 
				}
				stateTimeWeapon += Gdx.graphics.getDeltaTime(); 
			}
		}
	}

	protected void updateMove(OneRoom game){
		dx=0;
		dy=0;
		float realSpeed = speed-speedMalus;
		if(!onTarget()){
			float distX = targetX - x;
			float distY = targetY - y;
			if(distX != 0 && distY != 0){
				dx = Math.min((Math.abs(distX)/Math.abs(distY))*realSpeed,realSpeed);
				dy = Math.min(realSpeed - dx,realSpeed);
				if(distX<0){
					dx = -dx;
				}
				if(distY<0){
					dy = -dy;
				}
			}
		}

		if(dx != 0 || dy != 0){
			move = true;
			if(!game.getRoom().isBlocked(x+dx, y+dy)){
				this.x+=dx;
				this.y+=dy;
				message.move(dx, dy);

				if(dx>0 && dx >Math.abs(dy)){
					direction = DIRECTION_RIGHT;
				}else if(dx<0 && dx <-Math.abs(dy)){
					direction = DIRECTION_LEFT;
				}else if(dy<0 && dy <-Math.abs(dx)){
					direction = DIRECTION_DOWN;
				}else if(dy>0 && dy >Math.abs(dx)){
					direction = DIRECTION_UP;
				}
			}else if(!game.getRoom().isBlocked(x+realSpeed, y)){
				this.x+=realSpeed;
				message.move(realSpeed, 0);


				if(dx>0 && dx >Math.abs(dy)){
					direction = DIRECTION_RIGHT;
				}else if(dx<0 && dx <-Math.abs(dy)){
					direction = DIRECTION_LEFT;
				}else if(dy<0 && dy <-Math.abs(dx)){
					direction = DIRECTION_DOWN;
				}else if(dy>0 && dy >Math.abs(dx)){
					direction = DIRECTION_UP;
				}
			}else if(!game.getRoom().isBlocked(x, y+realSpeed)){
				this.y+=realSpeed;
				message.move(0, realSpeed);


				if(dx>0 && dx >Math.abs(dy)){
					direction = DIRECTION_RIGHT;
				}else if(dx<0 && dx <-Math.abs(dy)){
					direction = DIRECTION_LEFT;
				}else if(dy<0 && dy <-Math.abs(dx)){
					direction = DIRECTION_DOWN;
				}else if(dy>0 && dy >Math.abs(dx)){
					direction = DIRECTION_UP;
				}
			}
		}else{
			move = false;
		}
	}
	
	protected void updateAttack(OneRoom game){
		if(cooldownAttack<=0 && !onAttack && nearPlayer(game,nearPlayerDistance)){
			onAttack = true;
			stateTimeWeapon = 0f;
			cooldownAttack = maxCooldownAttack;
			shoot(game);
			GdxUtils.SOUND_PEASANT_ATTACK.play();
		}else if(onAttack && swordHitAnimation.isAnimationFinished(stateTimeWeapon)){
			onAttack = false;
		}else if(onAttack){
			// Test avec le heros
			if(nearPlayer(game,32)){
				game.getPlayer().touchByEnemy(game,this);
			}

		}else if(!onAttack && cooldownAttack>0){
			cooldownAttack--;
		}
	}
	
	/**
	 * @Override
	 * @param game
	 */
	public void die(OneRoom game){
		game.getFloorAnimations().add(new BodyPart(x, y, peasentTR[4][0],false));
		game.getFloorAnimations().add(new BodyPart(x, y, peasentTR[5][0],true));
		game.getFloorAnimations().add(new BodyPart(x, y, peasentTR[4][1],true));
		game.getFloorAnimations().add(new BodyPart(x, y, peasentTR[4][2],true));
		game.getHud().addKill(1);

		loot(game);
	}

	public void loot(OneRoom game){

		double rand = Math.random();
		if(rand>0.7){
			game.getItems().add(new Item(x, y, ITEM_TYPE.POTION, true));

		}else if(rand > 0.6){
			game.getItems().add(new Item(x, y, ITEM_TYPE.HEART, true));
		}else if(rand > 0.5){
			game.getItems().add(new Item(x, y, ITEM_TYPE.ENERGY, true));
		}
		
		if(rand < 0.08){
			game.getItems().add(new Item(x, y, ITEM_TYPE.BLACK_POTION, true));
		}

		game.getItems().add(new Item(x, y, ITEM_TYPE.XP_I, true));
		game.getItems().add(new Item(x, y, ITEM_TYPE.XP_I, true));
		game.getItems().add(new Item(x, y, ITEM_TYPE.XP_I, true));
		destroy = true;
	}

	public void setDelay(int delay){
		this.delay=delay;
	}

	public void shoot(OneRoom game){
		// Ne fait rien, c'est un paysan
	}
	
	
	protected void initUniqMessage(OneRoom game){
		Double random = Math.random();
		if(life==1){
			if(random<0.25){
				message.initMessage(x, y+28, "Be kind, I'm just a poor peasant.");
			}else if(random<0.5){
				message.initMessage(x, y+28, "I'm starting to panic.");
			}else{
				message.initMessage(x, y+28, msgs[(int)(Math.random()*msgs.length-1)]);
			}
		}else{
			message.initMessage(x, y+28, msgs[(int)(Math.random()*msgs.length-1)]);
		}
	}
	
	protected void writeMessage(OneRoom game){
		if(game.getPersonnages().size() == 1){
			initUniqMessage(game);
		}else{
			if(game.getPersonnages().size()<4 || Math.random()<0.2){
				message.initMessage(x, y+28, msgs[(int)(Math.random()*msgs.length-1)]);
			}
		}
		
	}
}
