package com.ldjam.characters;

import com.badlogic.gdx.Gdx;
import com.ldjam.game.Entity;
import com.ldjam.game.Message;
import com.ldjam.game.OneRoom;
import com.ldjam.game.Shoot;

public abstract class Personnage extends Entity {

	public static final int DIRECTION_DOWN = 0;
	public static final int DIRECTION_LEFT = 1;
	public static final int DIRECTION_RIGHT = 2;
	public static final int DIRECTION_UP = 3;
	
	protected int direction = DIRECTION_UP;
	protected float dx=0, dy=0, targetX=0, targetY=0, a=0, b=0;
	protected float speed=1, speedMalus;
	protected float stateTime; 
	protected boolean move = false;
	protected int attackDmg = 1;
	
	protected int cellWidth=24;
	protected int cellHeight=32;
	
	protected int life = 3;
	protected int maxLife = 3;
	protected int cooldownTouch = 0;
	protected float colorAfterHit;
	protected boolean destroy = false;
	protected Message message;

	
	public Personnage(float x, float y){
		super(x,y);
		this.message = new Message();
	}
	
	
	public void update(OneRoom game){
		if(cooldownTouch>0){
			cooldownTouch--;
		}
		if(colorAfterHit<0){
			colorAfterHit+=0.05f;
		}
		
		if(speedMalus>0){
			speedMalus-=(0.01*speed);
			if(speedMalus<0){
				speedMalus=0;
			}
		}
	}
	
	public void render(OneRoom game){
		// On met a jour le stateTime pour les animations
		stateTime += Gdx.graphics.getDeltaTime(); 
	}
	
	public boolean onTarget(){
		if(Math.abs(x-targetX)<4 && Math.abs(y-targetY)<4){
			return true;
		}
		return false;
	}
	
	public void setTarget(float targetX, float targetY){
		this.targetX = targetX;
		this.targetY = targetY;
	}
	
	public boolean nearPlayer(OneRoom game, float dist){
		if(Math.abs(game.getPlayer().getX()-this.x) <(dist/2) && Math.abs(game.getPlayer().getY()-this.y) <(dist/2)){
			return true;
		}
		return false;
	}
	
	public boolean touchByShoot(OneRoom game,Shoot s){
		if(life>0 && Math.abs(s.getX()-x)<(cellWidth/2) && Math.abs(s.getY()-y)<(cellHeight/2)){
			if(cooldownTouch<=0){
				if(game.getPlayer().getShootMode()>2){
					this.speedMalus = this.speed;
				}
				cooldownTouch = 25;
				colorAfterHit = 0;
				this.life--;
				if(life<=0){
					die(game);
					life = 0;
				}
			}
			return true;
		}
		return false;
	}


	/**
	 * @return the attackDmg
	 */
	public int getAttackDmg() {
		return attackDmg;
	}


	/**
	 * @param attackDmg the attackDmg to set
	 */
	public void setAttackDmg(int attackDmg) {
		this.attackDmg = attackDmg;
	}
	
	public abstract void die(OneRoom game);
	
	public boolean isDestroy(){
		return this.destroy;
	}
	
	/**
	 * @return the life
	 */
	public int getLife() {
		return life;
	}

	/**
	 * @param life the life to set
	 */
	public void setLife(int life) {
		this.life = life;
	}

	/**
	 * @return the maxLife
	 */
	public int getMaxLife() {
		return maxLife;
	}

	/**
	 * @param maxLife the maxLife to set
	 */
	public void setMaxLife(int maxLife) {
		this.maxLife = maxLife;
	}
	
	public float getSpeed() {
		return speed;
	}
	public void setSpeed(float speed) {
		this.speed = speed;
	}

	public Message getMessage(){
		return message;
	}
}
