package com.ldjam.game;

import com.badlogic.gdx.Gdx;
import com.badlogic.gdx.graphics.g2d.Animation;
import com.badlogic.gdx.graphics.g2d.BitmapFont;
import com.badlogic.gdx.graphics.g2d.TextureRegion;
import com.badlogic.gdx.graphics.g2d.Animation.PlayMode;
import com.badlogic.gdx.graphics.g2d.freetype.FreeTypeFontGenerator;
import com.badlogic.gdx.graphics.g2d.freetype.FreeTypeFontGenerator.FreeTypeFontParameter;

public class Hud {
	private BitmapFont font;
	private int kills=0, cptKill=0;
	private String strKill = "000";
	
	private String message;
	private float xMessage, yMessage;
	private int cptMessage;
	
	private int utilityIncrease = 0;
	private int attackIncrease = 0;
	private int increasePoint = 0;
	private boolean activateIncreaseMenu = false;
	private int dialboxSelec = 0;
	private Animation pressEscapeAnimation, newWaveAnimation, pressSpaceAnimation;
	private float stateTime = 0f;
	private boolean inHowTo = true, howToAttackNeeded=true, howToShieldNeeded=false;
	private int howToNeededDelay = 0;
	private boolean pause = false;
	
	
	private String[] utilityIncreaseDescription = {
			"Magic shield",
			"Faster in shield",
			"+2 MAX MP",
			"+1 MAX HP",
			"NOT YET",
			"NOT YET",
			"NOT YET",
			"NOT YET",
			"NOT YET",
			"NOT YET",
			"NOT YET",
			"NOT YET",
			"NOT YET",
			"NOT YET"
	};
	private String[] attackIncreaseDescription = {
			"Faster shoot",
			"Triple shoots",
			"More shoots",
			"Slow enemies",
			"NOT YET",
			"NOT YET",
			"NOT YET",
			"NOT YET",
			"NOT YET",
			"NOT YET",
			"NOT YET",
			"NOT YET",
			"NOT YET"
	};
	
	
	public Hud(){
		FreeTypeFontGenerator generator = new FreeTypeFontGenerator(Gdx.files.internal("hud/pixelOperator8.ttf"));
		FreeTypeFontParameter parameter = new FreeTypeFontParameter();
		parameter.size = 12;
		parameter.spaceX = 0;
		font = generator.generateFont(parameter);
		
		TextureRegion[][] regions = TextureRegion.split(GdxUtils.TEXTURE_PRESS_ESCAPE, 256, 16);
		TextureRegion[] frames = new TextureRegion[2];
		frames[0] = regions[0][0];
		frames[1] = regions[1][0];
		pressEscapeAnimation = new Animation(0.14f, frames);
		pressEscapeAnimation.setPlayMode(PlayMode.LOOP);
		
		TextureRegion[][] regions2 = TextureRegion.split(GdxUtils.TEXTURE_NEW_WAVE, 256, 16);
		TextureRegion[] frames2 = new TextureRegion[2];
		frames2[0] = regions2[0][0];
		frames2[1] = regions2[1][0];
		newWaveAnimation = new Animation(0.14f, frames2);
		newWaveAnimation.setPlayMode(PlayMode.LOOP);
		
		TextureRegion[][] regions3 = TextureRegion.split(GdxUtils.TEXTURE_PRESS_SPACE, 256, 16);
		TextureRegion[] frames3 = new TextureRegion[2];
		frames3[0] = regions3[0][0];
		frames3[1] = regions3[1][0];
		pressSpaceAnimation = new Animation(0.14f, frames3);
		pressSpaceAnimation.setPlayMode(PlayMode.LOOP);
	
	}
	
	/**
	 * Met a jour le HUD
	 * @param game
	 */
	public void update(OneRoom game){
		if(cptKill>0){
			cptKill--;
		}
		if(cptMessage>0){
			cptMessage--;
		}
		
		if(activateIncreaseMenu){
			updateIncreaseChoice(game);
		}

		if(inHowTo){
			updateHowTo(game);
		}
		
		if(game.getInput().isKeyEscapeJustPressed()){
			if(activateIncreaseMenu || (!activateIncreaseMenu && increasePoint>0)){
				activateIncreaseMenu = !activateIncreaseMenu;
			}else{
				pause = !pause;
			}
		}
		
		if(!inHowTo && howToNeededDelay>0){
			howToNeededDelay--;
			if(howToNeededDelay==0){
				inHowTo = true;
			}
		}
		
	}
	
	/**
	 * Dessine le HUD
	 * @param game
	 */
	public void render(OneRoom game){
		String strLevel = "";
		float lenghtXpBar = (
				((game.getPlayer().getXp()-game.getPlayer().getLevelLader()[game.getPlayer().getLevel()])*1.0f)/
				((game.getPlayer().getLevelLader()[game.getPlayer().getLevel()+1])-(game.getPlayer().getLevelLader()[game.getPlayer().getLevel()])*1.0f)
				)*25f;
		if(game.getPlayer().getLevel()<10){
			strLevel = "lvl : 0"+game.getPlayer().getLevel();
		}else{
			strLevel = "Lvl : "+game.getPlayer().getLevel();
		}
		
		if(increasePoint>0){
			font.setColor(0.1f,0.1f,0.1f,1);
			font.draw(game.getBatch(), "(+"+increasePoint+")", 434, 352);
			font.setColor(1f,0.5f,0.2f,1);
			font.draw(game.getBatch(), "(+"+increasePoint+")", 434, 353);

		}
		
		
		for(int i=0;i<game.getPlayer().getMaxLife();i++){
			game.getBatch().draw(i>=game.getPlayer().getLife()?GdxUtils.TEXTURE_LIFE_EMPTY:GdxUtils.TEXTURE_LIFE, 8+i*18, 340);
		}
		for(int i=0;i<game.getPlayer().getMaxEnergy();i++){
			game.getBatch().draw(i>=game.getPlayer().getEnergy()?GdxUtils.TEXTURE_HUD_ENERGY_EMPTY:GdxUtils.TEXTURE_HUD_ENERGY, 8+i*10, 324);
		}
		
		game.getBatch().draw(GdxUtils.TEXTURE_XP_BACK, 536, 340);
		game.getBatch().draw(GdxUtils.TEXTURE_XP_BAR, 541, 345,(int)lenghtXpBar,4);


		if(utilityIncrease>0){
			game.getBatch().draw(game.getInput().isKeyAltPressed()?GdxUtils.TEXTURE_ALT_PRESSED:GdxUtils.TEXTURE_ALT, 8, 8);
		}
		game.getBatch().draw(game.getInput().isKeySpacePressed()?GdxUtils.TEXTURE_SPACE_PRESSED:GdxUtils.TEXTURE_SPACE, 36, 8);
		
		
		game.getBatch().draw(cptKill>0?GdxUtils.TEXTURE_SKULL_ADD:GdxUtils.TEXTURE_SKULL, 580, 336);
		font.setColor(0.1f,0.1f,0.1f,1);
		font.draw(game.getBatch(), strKill, 604, 352);
		font.draw(game.getBatch(), strLevel, 480, 352);

		font.setColor(0.9f,0.9f,0.9f,1);
		font.draw(game.getBatch(), strKill, 604, 353);
		font.draw(game.getBatch(), strLevel, 480, 353);

		font.setColor(1f,1f,1f,1);
		
		if(cptMessage>0){
			if(cptMessage<30){
				font.setColor(0.1f,0.1f,0.1f,0.6f);
			}else if(cptMessage<15){
				font.setColor(0.1f,0.1f,0.1f,0.3f);
			}else{
				font.setColor(0.1f,0.1f,0.1f,1f);
			}
			font.draw(game.getBatch(), message, xMessage, yMessage-cptMessage);
			
			if(cptMessage<30){
				font.setColor(0.9f,0.9f,0.9f,0.6f);
			}else if(cptMessage<15){
				font.setColor(0.9f,0.9f,0.9f,0.3f);
			}else{
				font.setColor(0.9f,0.9f,0.9f,1f);
			}
			font.draw(game.getBatch(), message, xMessage, yMessage-cptMessage+1);
			font.setColor(1f,1f,1f,1);
		}
		
		if(!activateIncreaseMenu && !pause){
			if(game.getPersonnages().isEmpty() && game.getCptWaves() < 180){
				game.getBatch().draw(newWaveAnimation.getKeyFrame(stateTime), 192,2);
			}else if(increasePoint>0){
				game.getBatch().draw(pressEscapeAnimation.getKeyFrame(stateTime), 192,2);
			}
		}
		
		if(pause){
			game.getBatch().draw(GdxUtils.TEXTURE_PAUSE, 160,120);
		}
		
		if(activateIncreaseMenu){
			renderIncreaseChoice(game);
		}
		
		if(inHowTo){
			renderHowTo(game);
		}
		stateTime += Gdx.graphics.getDeltaTime(); 


	}
	
	public void addKill(int val){
		this.kills+=val;
		strKill = String.valueOf(kills);
		while(strKill.length()<3){
			strKill = "0"+strKill;
		}
		cptKill=25;
	}
	
	public void initMessage(float x, float y, String msg){
		this.cptMessage = 200;
		this.xMessage = x;
		this.yMessage = y+200;
		this.message = msg;
	}
	
	public boolean onPause(){
		return activateIncreaseMenu || inHowTo || pause;
	}

	
	public void renderIncreaseChoice(OneRoom game){
		game.getBatch().draw(GdxUtils.TEXTURE_DIALBOX, 128, 256);
		game.getBatch().draw(GdxUtils.TEXTURE_DIALBOX_SELEC, 142, dialboxSelec==0?282:264);
		font.draw(game.getBatch(), "You have gained a level.\nWhat power would you like to increase? ",  132, 330);
		if(attackIncrease<4){
			font.draw(game.getBatch(), "Attack ("+attackIncreaseDescription[attackIncrease]+")", 142, 298);
		}else{
			font.draw(game.getBatch(), "--------", 142, 298);
		}
		if(utilityIncrease<4){
			font.draw(game.getBatch(), "Utility ("+utilityIncreaseDescription[utilityIncrease]+")", 142, 280);
		}else{
			font.draw(game.getBatch(), "--------", 142, 280);
		}
	}
	
	
	public void updateIncreaseChoice(OneRoom game){
		if(game.getInput().isKeyDownPressed() && dialboxSelec==0){
			dialboxSelec=1;
			GdxUtils.SOUND_DIALBOX_SELEC.play();

		}else if(game.getInput().isKeyUpPressed() && dialboxSelec==1){
			dialboxSelec=0;
			GdxUtils.SOUND_DIALBOX_SELEC.play();
			
		}else if(game.getInput().isKeySpaceJustPressed()){
			boolean increase = true;
			if(dialboxSelec==0){
				if(attackIncrease<4){
					attackIncrease++;
					if(attackIncrease==1){
						game.getPlayer().increaseAttack1(game);
					}else if(attackIncrease==2){
						game.getPlayer().increaseAttack2(game);
					}else if(attackIncrease==3){
						game.getPlayer().increaseAttack3(game);
					}else if(attackIncrease==4){
						game.getPlayer().increaseAttack4(game);
					}
				}else{
					increase = false;
				}
			}else{
				if(utilityIncrease<4){
					utilityIncrease++;
					if(utilityIncrease==1){
						this.howToShieldNeeded=true;
						this.howToNeededDelay=15;
					}else if(utilityIncrease==2){
						game.getPlayer().fasterInShield(1.5f);
					}else if(utilityIncrease==3){
						game.getPlayer().setMaxEnergy(game.getPlayer().getMaxEnergy()+2);
						game.getPlayer().setEnergy(game.getPlayer().getEnergy()+2);
					}else if(utilityIncrease==4){
						game.getPlayer().setMaxLife(game.getPlayer().getMaxLife()+1);
						game.getPlayer().setLife(game.getPlayer().getLife()+1);
					}
				}else{
					increase = false;
				}
			}
			if(increase){
				GdxUtils.SOUND_DIALBOX_ACTIVATE.play();
				increasePoint--;
				activateIncreaseMenu = false;
			}else{
				GdxUtils.SOUND_DIALBOX_ERROR.play();
			}
			
		}
	}
	
	private void renderHowTo(OneRoom game){
		if(howToAttackNeeded){
			game.getBatch().draw(GdxUtils.TEXTURE_HOWTO_SHOOT, 160, 140);
		}else if(howToShieldNeeded){
			game.getBatch().draw(GdxUtils.TEXTURE_HOWTO_SHIELD, 160, 140);
		}
		game.getBatch().draw(pressSpaceAnimation.getKeyFrame(stateTime), 192,2);
	}
	
	private void updateHowTo(OneRoom game){
		if(game.getInput().isKeySpaceJustPressed()){
			if(howToAttackNeeded){
				this.howToAttackNeeded=false;
			}
			if(howToShieldNeeded){
				this.howToShieldNeeded=false;
			}
			this.inHowTo=false;
			GdxUtils.SOUND_DIALBOX_ACTIVATE.play();
		}
	}
	
	
	public void addIncreasePoint(){
		this.increasePoint++;
	}
	
	public int getAttackIncrease(){
		return attackIncrease;
	}
	
	public int getUtilityIncrease(){
		return utilityIncrease;
	}
	
	public BitmapFont getFont(){
		return font;
	}
	
	public int getKills(){
		return kills;
	}
	
	
}
