package com.ldjam.game;

import com.badlogic.gdx.graphics.g2d.TextureRegion;
import com.ldjam.game.Entity;
import com.ldjam.game.OneRoom;

public class Item extends Entity {

	protected TextureRegion currentFrame;
	protected static TextureRegion[][] itemFrames;
	protected float dx, dy;
	protected int duration;
	protected int duration2;
	protected ITEM_TYPE type;
	protected boolean taken = false;

	public enum ITEM_TYPE {POTION, HEART, ENERGY, SPEED, XP_I, XP_V, BLACK_POTION};

	public Item(float x, float y, ITEM_TYPE type, boolean move) {
		super(x, y);
		if(move){
			this.dx = (float) ((Math.random()*4)-2);
			this.dy = (float) ((Math.random()*4)-2);

		}else{
			this.dx = 0;
			this.dy = 0;
		}
		duration =(int)(Math.random()*32+16);
		duration2 =(int)(Math.random()*2048+1024);
		if(!move){
			duration2 = 3*duration2;
		}
		this.type = type;
		if(itemFrames == null){
			itemFrames = TextureRegion.split(GdxUtils.TEXTURE_ITEMS, 16, 16);
		}
		switch (type) {
		case POTION:
			currentFrame = itemFrames[0][0];
			break;
		case HEART:
			currentFrame = itemFrames[0][1];
			break;
		case ENERGY:
			currentFrame = itemFrames[1][0];
			break;
		case SPEED:
			currentFrame = itemFrames[1][1];
			break;
		case XP_I:
			currentFrame = itemFrames[2][0];
			break;
		case XP_V:
			currentFrame = itemFrames[2][1];
			break;
		case BLACK_POTION:
			currentFrame = itemFrames[3][0];
			break;
		}

	}

	public void update(OneRoom game){
		if(duration>0){
			duration--;
			if(!game.getRoom().isBlockedRestricted(x+dx, y+dy)){
				x+=dx;
				y+=dy;
			}
		}else if(duration2>0){
			duration2--;
			if(this.nearPlayer(game, 24)){
				takeObject(game);
			}
		}

	}

	public void render(OneRoom game){
		if(duration2<35){
			game.getBatch().setColor(1, 1, 1, 0.75f);
		}else if(duration2<20){
			game.getBatch().setColor(1, 1, 1, 0.5f);
		}else if(duration2<10){
			game.getBatch().setColor(1, 1, 1, 0.25f);
		}
		game.getBatch().draw(currentFrame, x-8, y-8);
		game.getBatch().setColor(1, 1, 1,1f);
	}

	public boolean nearPlayer(OneRoom game, float dist){
		if(Math.abs(game.getPlayer().getX()-this.x) <(dist/2) && Math.abs(game.getPlayer().getY()-8-this.y) <(dist/2)){
			return true;
		}
		return false;
	}

	public void takeObject(OneRoom game){
		String msg = null;
		switch(type){
		case POTION:
			if(game.getPlayer().getLife()<game.getPlayer().getMaxLife()){
				game.getPlayer().setLife(game.getPlayer().getLife()+1);
				msg="REGEN 1 HP";
				game.addAnimation(new SimpleAnimation(game.getPlayer().getX()-16, game.getPlayer().getY()-16, game.getPlayer().getHealAnimation()));
				GdxUtils.SOUND_ITEM_POTION.play();
			}else{
				GdxUtils.SOUND_ITEM_PICKUP.play();
			}
			break;
		case HEART:
			game.getPlayer().setMaxLife(game.getPlayer().getMaxLife()+1);
			game.getPlayer().setLife(game.getPlayer().getLife()+1);
			msg="+1 MAX HP";
			GdxUtils.SOUND_ITEM_PICKUP.play();
			break;
		case ENERGY:
			game.getPlayer().setMaxEnergy(game.getPlayer().getMaxEnergy()+1);
			game.getPlayer().setEnergy(game.getPlayer().getEnergy()+1);
			msg="+1 MAX MP";
			GdxUtils.SOUND_ITEM_PICKUP.play();
			break;
		case SPEED:
			game.getPlayer().setSpeed(game.getPlayer().getSpeed()+0.1f);
			msg="+1 MOVE SPEED";
			game.addAnimation(new SimpleAnimation(game.getPlayer().getX()-16, game.getPlayer().getY()-16, game.getPlayer().getSpeedPotionAnimation()));
			GdxUtils.SOUND_ITEM_POTION.play();
			break;
		case XP_I:
			game.getPlayer().addXp(game,1);
			msg="+1 XP";
			GdxUtils.SOUND_ITEM_PICKUP.play();
			break;
		case XP_V:
			game.getPlayer().addXp(game,5);
			msg="+5 XP";
			GdxUtils.SOUND_ITEM_PICKUP.play();
			break;
		case BLACK_POTION:
			game.getPlayer().addDarkPotion(game);
			msg="THE DARK POTION !";
			GdxUtils.SOUND_ITEM_POTION.play();
			break;
		}
		if(msg != null){
			game.getHud().initMessage(game.getPlayer().getX()-64, game.getPlayer().getY(), msg);
		}
		taken = true;
	}

	public boolean isDestroy(){
		return taken || (duration<=0 && duration2<=0);
	}

}
