package com.ldjam.game;

import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;

import com.badlogic.gdx.ApplicationAdapter;
import com.badlogic.gdx.Gdx;
import com.badlogic.gdx.graphics.GL20;
import com.badlogic.gdx.graphics.g2d.SpriteBatch;
import com.ldjam.characters.Personnage;
import com.ldjam.characters.BlackSoldier;
import com.ldjam.characters.GoodWizard;
import com.ldjam.characters.GreenWizard;
import com.ldjam.characters.Peasant;
import com.ldjam.characters.PeasantSoldier;

public class OneRoom extends ApplicationAdapter {
	SpriteBatch batch;
	private Room room;
	private List<Personnage> personnages;
	private List<Item> items;
	private List<GameAnimation> animations,floorAnimations, tmpAnimations;
	private Player player;
	private Input input;
	private Hud hud;
	private TitleMenu title;
	
	private int waves = 0;
	private int cptWaves = 0;
	private Intro intro;
	private EndGame endGame;
	
	private static final int END_GAME_WAVE = 12;

	@Override
	public void create () {
		batch = new SpriteBatch();
		room = new Room();
		personnages = new ArrayList<Personnage>();
		animations = new ArrayList<GameAnimation>();
		tmpAnimations = new ArrayList<GameAnimation>();
		floorAnimations = new ArrayList<GameAnimation>();
		items = new ArrayList<Item>();
		player = new Player(320,300);
		input = new Input();
		hud = new Hud();
		title = new TitleMenu();
		cptWaves = 200;
		intro = new Intro();
		endGame = new EndGame();
		
//		for(int i=0;i<500;i++){
//			player.addXp(this, 1);
//		}
//		player.setMaxEnergy(player.getMaxEnergy()+4);
//		player.setEnergy(player.getEnergy()+4);
//		
//		player.setMaxLife(player.getMaxLife()+5);
//		player.setLife(player.getLife()+5);
	}
	
	public void initGame(){
		personnages = new ArrayList<Personnage>();
		animations = new ArrayList<GameAnimation>();
		tmpAnimations = new ArrayList<GameAnimation>();
		floorAnimations = new ArrayList<GameAnimation>();
		items = new ArrayList<Item>();
		player = new Player(320,300);
		input = new Input();
		hud = new Hud();
		cptWaves = 200;
		waves=0;
		intro = new Intro();
		endGame = new EndGame();
		

	}

	@Override
	public void render () {
		this.update();

		Gdx.gl.glClearColor(1, 0, 0, 1);
		Gdx.gl.glClear(GL20.GL_COLOR_BUFFER_BIT);
		batch.begin();

		room.render(this);

		if(title.isActive()){
			title.render(this);
		}else if(!intro.isFinish()){
			intro.render(this);
			player.render(this);
			room.renderFront(this);
		}else{
			for (GameAnimation a : floorAnimations) {
				a.render(this);
			}

			for (Item it : items) {
				it.render(this);
			}

			player.render(this);
			for (Personnage p : personnages) {
				p.render(this);
			}

			for (GameAnimation a : animations) {
				a.render(this);
			}
			
			for (Personnage p : personnages) {
				p.getMessage().render(this);
			} 
			room.renderFront(this);
			hud.render(this);
			
			if(waves == END_GAME_WAVE && !endGame.isFinish()){
				endGame.render(this);
			}
			if(player.getLife()<=0){
				title.renderGameOver(this);
			}
		}

		batch.end();
	}

	public void update(){
		input.update(this);

		if(title.isActive()){
			title.update(this);
		}else if(!intro.isFinish()){
			intro.update(this);
		}else if(waves==END_GAME_WAVE && !endGame.isFinish()){
			endGame.update(this);
		}else{
			if(!hud.onPause()){
				room.update(this);
				player.update(this);

				Iterator<Personnage> itP = personnages.iterator();
				while (itP.hasNext()) {
					Personnage p = itP.next();
					if(p.isDestroy()){
						itP.remove();
					}else{
						p.update(this);
					}
				}

				Iterator<GameAnimation> it = animations.iterator();
				while (it.hasNext()) {
					GameAnimation ga = it.next();
					if(ga.isFinish()){
						it.remove();
					}else{
						ga.update(this);
					}
				}
				Iterator<GameAnimation> it2 = floorAnimations.iterator();
				while (it2.hasNext()) {
					GameAnimation ga = it2.next();
					if(ga.isFinish()){
						it2.remove();
					}else{
						ga.update(this);
					}
				}
				Iterator<Item> itItem = items.iterator();
				while (itItem.hasNext()) {
					Item item = itItem.next();
					if(item.isDestroy()){
						itItem.remove();
					}else{
						item.update(this);
					}
				}
				for (GameAnimation gameAnimation : tmpAnimations) {
					animations.add(gameAnimation);
				}
				tmpAnimations.clear();

				if(personnages.isEmpty()){
					if(cptWaves>0){
						cptWaves--;
					}else{
						addNewEnemies();
					}
				}
				
			}
			if(player.getLife()<=0){
				title.updateGameOver(this);
			}
			hud.update(this);
			
			if(waves > 5 && GdxUtils.MUSIC_01.isLooping()){
				GdxUtils.MUSIC_01.setLooping(false);
			}if(waves > 6 && !GdxUtils.MUSIC_02.isPlaying() && !GdxUtils.MUSIC_01.isPlaying()){
				GdxUtils.MUSIC_02.play();
				GdxUtils.MUSIC_02.setLooping(true);
			}
		}
	}

	public SpriteBatch getBatch(){
		return batch;
	}

	public Input getInput(){
		return input;
	}

	public Room getRoom(){
		return room;
	}

	public Player getPlayer(){
		return player;
	}

	public List<Personnage> getPersonnages(){
		return personnages;
	}

	public List<GameAnimation> getAnimations(){
		return animations;
	}
	public void addAnimation(GameAnimation a){
		this.tmpAnimations.add(a);
	}
	public List<GameAnimation> getFloorAnimations(){
		return floorAnimations;
	}
	public List<Item> getItems(){
		return items;
	}

	public Hud getHud(){
		return hud;
	}
	
	public int getCptWaves(){
		return cptWaves;
	}
	
	public TitleMenu getTitle(){
		return title;
	}
	
	public void addNewEnemies(){
		cptWaves = 500;
		waves++;
		int initY = 34;
		if(waves == 1){
			GdxUtils.MUSIC_01.play();
			GdxUtils.MUSIC_01.setLooping(true);
//			personnages.add(new GreenWizard(320, 32));
			personnages.add(new Peasant(314+(int)(Math.random()*12), initY));
//			personnages.add(new Peasant(320, 32));
//			personnages.add(new PeasantSoldier(320, 32));
//			personnages.add(new BlackSoldier(320, 32));
		}else if(waves == 2){
			Peasant p1 = new Peasant(314+(int)(Math.random()*12), initY);
			Peasant p2 = new Peasant(314+(int)(Math.random()*12), 32);
			p2.setDelay(120);
			Peasant p3 = new Peasant(314+(int)(Math.random()*12), initY);
			p3.setDelay(240);

			personnages.add(p1);
			personnages.add(p2);
			personnages.add(p3);

		}else if(waves == 2){
			Peasant p1 = new Peasant(320, initY);
			Peasant p2 = new Peasant(320, initY);
			p2.setDelay(10);
			Peasant p3 = new PeasantSoldier(320, initY);
			p3.setDelay(150);
			Peasant p4 = new Peasant(320, initY);
			p3.setDelay(350);

			personnages.add(p1);
			personnages.add(p2);
			personnages.add(p3);
			personnages.add(p4);
		}else if(waves == 3){
			Peasant p1 = new PeasantSoldier(320, initY);
			Peasant p2 = new PeasantSoldier(320, initY);
			p2.setDelay(32);
			Peasant p3 = new Peasant(320, initY);
			p3.setDelay(188);
			Peasant p4 = new Peasant(320, initY);
			p4.setDelay(194);
			Peasant p5 = new Peasant(320, initY);
			p5.setDelay(198);

			personnages.add(p1);
			personnages.add(p2);
			personnages.add(p3);
			personnages.add(p4);
			personnages.add(p5);
		}else if(waves == 4){
			Peasant p1 = new BlackSoldier(320, initY);
			personnages.add(p1);
		}else if(waves == 5){
			Peasant p1 = new Peasant(320, initY);
			Peasant p2 = new Peasant(320, initY);
			p2.setDelay(10);
			Peasant p3 = new PeasantSoldier(320, initY);
			p3.setDelay(150);
			Peasant p4 = new BlackSoldier(320, initY);
			p4.setDelay(180);
			personnages.add(p1);
			personnages.add(p2);
			personnages.add(p3);
			personnages.add(p4);
		}else if(waves == 6){
			Peasant p1 = new BlackSoldier(320, initY);
			Peasant p2 = new PeasantSoldier(320, initY);
			p2.setDelay(10);
			Peasant p3 = new PeasantSoldier(320, initY);
			p3.setDelay(150);
			Peasant p4 = new BlackSoldier(320, initY);
			p4.setDelay(180);
			personnages.add(p1);
			personnages.add(p2);
			personnages.add(p3);
			personnages.add(p4);
		}else if(waves == 6){
			Peasant p1 = new PeasantSoldier(320, initY);
			Peasant p2 = new PeasantSoldier(320, initY);
			p2.setDelay(10);
			Peasant p3 = new PeasantSoldier(320, initY);
			p3.setDelay(50);
			Peasant p4 = new PeasantSoldier(320, initY);
			p4.setDelay(50);
			Peasant p5 = new PeasantSoldier(320, initY);
			p5.setDelay(280);
			personnages.add(p1);
			personnages.add(p2);
			personnages.add(p3);
			personnages.add(p4);
			personnages.add(p5);
		}else if(waves == 7){
			personnages.add(new GoodWizard(320, initY));
			Peasant p1 = new Peasant(320, initY);
			Peasant p2 = new Peasant(320, initY);
			p1.setDelay(400);
			p2.setDelay(650);
			personnages.add(p1);
			personnages.add(p2);
		}else if(waves == 8){
			personnages.add(new GoodWizard(320, initY));
			Peasant p1 = new PeasantSoldier(320, initY);
			Peasant p2 = new GoodWizard(320, initY);
			Peasant p3 = new Peasant(320, initY);
			Peasant p4 = new Peasant(320, initY);
			p1.setDelay(400);
			p2.setDelay(1050);
			p3.setDelay(1250);
			p4.setDelay(1350);
			personnages.add(p1);
			personnages.add(p2);
			personnages.add(p3);
			personnages.add(p4);
		}else if(waves == 9){
			for(int i=0;i<15;i++){
				Peasant p = new Peasant(320, initY);
				p.setDelay(i*60);
				personnages.add(p);
			}
		}else if(waves == 10){
			personnages.add(new GreenWizard(320, initY));

		}else if(waves == 11){
			for(int i=0;i<8;i++){
				Peasant p = new Peasant(320, initY);
				p.setDelay(i*50);
				Peasant p2 = new PeasantSoldier(320, initY);
				p2.setDelay(i*70);
				personnages.add(p);
				personnages.add(p2);
			}
			Peasant p1 = new BlackSoldier(320, initY);
			p1.setDelay(600);
			personnages.add(p1);
		}else if(waves == 12){
			Peasant p1 = new BlackSoldier(320, initY);
			p1.setDelay(10);
			Peasant p2 = new GoodWizard(320, initY);
			p2.setDelay(200);
			Peasant p3 = new GreenWizard(320, initY);
			p3.setDelay(600);
			personnages.add(p1);
			personnages.add(p2);
			personnages.add(p3);
		}else if(waves == 12){
			for(int i=0;i<30;i++){
				Peasant p = new Peasant(320, initY);
				p.setDelay(i*50);
				personnages.add(p);
				if(i%5==0){
					Peasant p2 = new PeasantSoldier(320, initY);
					p2.setDelay(i*70);
					personnages.add(p2);
				}
				if(i%7==0){
					Peasant p1 = new BlackSoldier(320, initY);
					p1.setDelay(600);
					personnages.add(p1);
				}
			}
		}else if(waves == 13){
			for(int i=0;i<100;i++){
				Peasant p = new Peasant(320, initY);
				p.setDelay(i*60);
				personnages.add(p);
				if(i%3==0){
					Peasant p2 = new PeasantSoldier(320, initY);
					p2.setDelay(i*85);
					personnages.add(p2);
				}
				if(i%5==0){
					Peasant p1 = new BlackSoldier(320, initY);
					p1.setDelay(i*90);
					personnages.add(p1);
				}
				if(i%7==0){
					Peasant p1 = new GoodWizard(320, initY);
					p1.setDelay(i*65);
					personnages.add(p1);
				}
				if(i%9==0){
					Peasant p1 = new GreenWizard(320, initY);
					p1.setDelay(i*100);
					personnages.add(p1);
				}
			}
		}
		
	}
}
