package com.ldjam.game;

import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;

import com.badlogic.gdx.graphics.g2d.Animation;
import com.badlogic.gdx.graphics.g2d.TextureRegion;
import com.badlogic.gdx.graphics.g2d.Animation.PlayMode;
import com.ldjam.characters.Personnage;

public class Player extends Personnage{

	protected final int decalStep = 12;

	protected TextureRegion currentFrame;
	protected TextureRegion[][] frames, energyFrames, shieldFrames;
	protected float speed, shootSpeed;
	protected List<Point> blockpoint;
	protected List<Shoot> shoots;
	protected Animation energyLoad, energyLoaded, shieldLoad, shieldLoaded;
	protected int loadEnergy=0, loadShield;
	
	protected int decalShootX = 0;
	protected int decalShootY = 12;
	protected int cptLoadEnergy = 0;
	protected int cptLoadShield = 0;
	protected int cptDecal;
	protected int cptDarkPotion=0;
	
	protected int energy, maxEnergy, cptUseEnergy, cptRecupEnergy;
	
	protected Animation[] impactAnimations, impactPlayerAnimations;

	protected Animation levelUpAnimation, healAnimation, speedPotionAnimation;
	
	protected int level = 1;
	protected int xp = 0;
	protected float boostInShield = 1f;
	protected int shootMode = 0;
	protected int[] levelLader = {0,0,10,25,50,80,120,200,300,400,500,99999999};
	protected boolean dead = false;
	
	
	/**
	 * Constructeur
	 * @param x
	 * @param y
	 */
	public Player(float x, float y){
		super(x,y);
		direction = DIRECTION_DOWN;
		frames = TextureRegion.split(GdxUtils.TEXTURE_WIZARD, 64, 64);
		energyFrames = TextureRegion.split(GdxUtils.TEXTURE_ENERGY, 16, 16);
		shieldFrames = TextureRegion.split(GdxUtils.TEXTURE_SHIELD, 32, 32);
		speed = 2.5f;
		shootSpeed=4f;
		blockpoint = new ArrayList<Point>();
		blockpoint.add(new Point(x,y));
//		blockpoint.add(new Point(x+16,y+16));
//		blockpoint.add(new Point(x-16,y+16));
//		blockpoint.add(new Point(x+16,y-16));
//		blockpoint.add(new Point(x-16,y-16));
		
		shoots = new ArrayList<Shoot>();
		
		energyLoad = GdxUtils.loadAnimation(energyFrames, 5, 0, 0.08f);
		energyLoad.setPlayMode(PlayMode.NORMAL);
		energyLoaded = GdxUtils.loadAnimation(energyFrames, 6, 1, 0.08f);
		energyLoaded.setPlayMode(PlayMode.LOOP);
		
		shieldLoad = GdxUtils.loadAnimation(shieldFrames, 4, 0, 0.08f);
		shieldLoad.setPlayMode(PlayMode.NORMAL);
		shieldLoaded = GdxUtils.loadAnimation(shieldFrames, 4, 1, 0.08f);
		shieldLoaded.setPlayMode(PlayMode.LOOP);

		maxLife = 3;
		life = maxLife;
		
		energy = 5;
		maxEnergy = 5;
		
		TextureRegion[][] impactsFrames = TextureRegion.split(GdxUtils.TEXTURE_EVILWIZARD_IMPACT, 32, 32);
		impactAnimations = new Animation[4];
		for(int i=0;i<4;i++){
			impactAnimations[i] = GdxUtils.loadAnimation(impactsFrames, 6, i, 0.04f);
			impactAnimations[i].setPlayMode(PlayMode.NORMAL);
		}
		
		TextureRegion[][] impactsPlayerFrames = TextureRegion.split(GdxUtils.TEXTURE_PLAYER_IMPACT, 32, 32);
		impactPlayerAnimations = new Animation[4];
		for(int i=0;i<4;i++){
			impactPlayerAnimations[i] = GdxUtils.loadAnimation(impactsPlayerFrames, 6, i, 0.04f);
			impactPlayerAnimations[i].setPlayMode(PlayMode.NORMAL);
		}
		
		TextureRegion[][] lvlUpTextureRegion = TextureRegion.split(GdxUtils.TEXTURE_LEVELUP, 32, 32);
		levelUpAnimation = GdxUtils.loadAnimation(lvlUpTextureRegion, 8, 0, 0.08f);
		healAnimation = GdxUtils.loadAnimation(lvlUpTextureRegion, 8, 1, 0.06f);
		speedPotionAnimation = GdxUtils.loadAnimation(lvlUpTextureRegion, 8, 2, 0.06f);
		dead=false;
		
	}
	
	/**
	 * Met a jour l'objet
	 */
	public void update(OneRoom game){
		super.update(game);
		if(life>0){
			controls(game);
		}
		
		Iterator<Shoot> it = shoots.iterator();
		while(it.hasNext()){
			Shoot s = it.next();
			if(s.isDestroy()){
				it.remove();
			}else{
				s.update(game);
				s.touchCharacter(game);
			}
		}
		cptDecal++;
		if(cptDecal>2*decalStep){
			cptDecal=0;
		}
		if(cooldownTouch>0){
			cooldownTouch--;
		}
		if(cptDarkPotion>0){
			cptDarkPotion--;
			if(cptDarkPotion%15==0){
				game.addAnimation(new SimpleAnimation(x-16,y-16,getRandomImpactAnimation()));
			}
			if(cptDarkPotion%20==0){
				randomShoot(game);
			}
		}
	}
	
	/**
	 * Affiche l'objet
	 */
	public void render(OneRoom game){
		game.getBatch().draw(GdxUtils.TEXTURE_SHADOW, x-8, y-32);
		game.getBatch().draw(frames[direction][loadEnergy], x-32, y-32-(cptDecal>decalStep?1:0));
		if(loadEnergy>0 && direction != DIRECTION_UP){
			if(loadEnergy==1){
				game.getBatch().draw(energyLoad.getKeyFrame(stateTime), x-(direction==DIRECTION_DOWN?8:0), y-decalShootY-(cptDecal>decalStep?1:0));
			}else{
				game.getBatch().draw(energyLoaded.getKeyFrame(stateTime), x-(direction==DIRECTION_DOWN?8:0), y-decalShootY-(cptDecal>decalStep?1:0));
			}
		}
		if(loadShield>0){
			if(loadShield==1){
				game.getBatch().draw(shieldLoad.getKeyFrame(stateTime), x-16, y-16-(cptDecal>decalStep?1:0));
			}else{
				game.getBatch().draw(shieldLoaded.getKeyFrame(stateTime), x-16, y-16-(cptDecal>decalStep?1:0));
			}
		}
		for (Shoot shoot : shoots) {
			shoot.render(game);
		}
		super.render(game);
	}
	
	/**
	 * BOUGER
	 * @param game
	 */
	public void controls(OneRoom game){
		
		// Gestion du shoot
		controlShoot(game);
		
		// Gestion du bouclier (disponible a partir d'une augmentation)
		if(game.getHud().getUtilityIncrease()>0){
			controlShield(game);
		}
		
		// Gestion des deplacements
		controlMove(game);
	}
	
	/**
	 * Gestion du tir
	 * @param game
	 */
	private void controlShoot(OneRoom game){
		// Gestion du shoot
		if(loadShield==0 && game.getInput().isKeySpaceJustPressed()){
			// Nouvel appui sur espace
			stateTime=0f;
			loadEnergy=1;
			
			GdxUtils.SOUND_ENERGY_LOAD.play();
		}else if(game.getInput().isKeySpacePressed()){
			if((loadEnergy==1 && energyLoad.isAnimationFinished(stateTime))){
				loadEnergy++;
				cptLoadEnergy=1;
				GdxUtils.SOUND_ENERGY_OK.play(0.6f);
			}else if(loadEnergy==2){
				cptLoadEnergy++;
				if(cptLoadEnergy%4==0 && loadEnergy<3){
					loadEnergy++;
				}
			}
		}else{
			if(loadEnergy>=3){
				shoot(game);
			}
			loadEnergy=0;
		}
	}
	
	/**
	 * Gestion du bouclier
	 * @param game
	 */
	private void controlShield(OneRoom game){
		if(game.getInput().isKeyAltJustPressed()){
			loadShield=1;
			stateTime=0f;
			GdxUtils.SOUND_SHIELD_LOAD.play();
		}else if(game.getInput().isKeyAltPressed() && energy>0){
			if((loadShield==1 && shieldLoad.isAnimationFinished(stateTime))){
				loadShield++;
				cptLoadShield=1;
				
			}else if(loadShield==2){
				cptLoadShield++;
			}
			if(cptUseEnergy>25){
				energy--;
				GdxUtils.SOUND_SHIELD_USE_ENERGY.play();
				cptUseEnergy=0;
			}else{
				cptUseEnergy++;
			}
		}else{
			loadShield=0;
			if(cptRecupEnergy>75){
				if(energy<maxEnergy)energy++;
				cptRecupEnergy=0;
			}else{
				cptRecupEnergy++;
			}
		}
	}
	
	/**
	 * Gestion des deplacements
	 * @param game
	 */
	private void controlMove(OneRoom game){
		int newDirection = direction;
		float realSpeed = speed;
		if(cptDarkPotion>0){
			realSpeed = realSpeed*1.5f;
		}
		dx=0;
		dy=0;
		
		// Gestion de la direction
		if(game.getInput().isKeyUpPressed()){
//			if(!game.getInput().isKeySpacePressed()){
				newDirection = DIRECTION_UP;
				decalShootX = 0;
				decalShootY = 12;
//			}
			dy=realSpeed;
		}else if(game.getInput().isKeyDownPressed()){
			newDirection = DIRECTION_DOWN;
//			if(!game.getInput().isKeySpacePressed()){
				decalShootX = 0;
				decalShootY = 12;
//			}
			dy=-realSpeed;
		}
		if(game.getInput().isKeyLeftPressed()){
//			if(!game.getInput().isKeySpacePressed()){
				newDirection = DIRECTION_LEFT;
				decalShootX = 16;
				decalShootY = 12;
//			}
			dx=-realSpeed;
		}else if(game.getInput().isKeyRightPressed()){
//			if(!game.getInput().isKeySpacePressed()){
				newDirection = DIRECTION_RIGHT;
				decalShootX = 0;
				decalShootY = 12;
//			}
			dx=+realSpeed;
		}
		if(direction != newDirection){
			direction = newDirection;
			// Changement de direction
		}
		
		
		
		if(dx != 0 || dy !=0){
			if(dx != 0 && dy != 0){
				dx = dx*0.66f;
				dy = dy*0.66f;
			}
			if(loadShield>0){
				dx = dx*boostInShield;
				dy = dy*boostInShield;
			}
			if(!game.getRoom().isBlocked(blockpoint,dx, dy)){
				x+=dx;
				y+=dy;
				for (Point point : blockpoint) {
					point.setX(point.getX()+dx);
					point.setY(point.getY()+dy);
				}
			}else if(!game.getRoom().isBlocked(blockpoint,0, dy)){
				y+=dy;
				for (Point point : blockpoint) {
					point.setY(point.getY()+dy);
				}
			}else if(!game.getRoom().isBlocked(blockpoint,dx, 0)){
				x+=dx;
				for (Point point : blockpoint) {
					point.setX(point.getX()+dx);
				}
			}
		}
	}
	
	/**
	 * TIR !
	 * @param game
	 */
	public void shoot(OneRoom game){
		Shoot newShoot = null;
		Shoot newShoot2 = null;
		Shoot newShoot3 = null;
		Shoot newShoot4 = null;
		Shoot newShoot5 = null;

		if(direction == DIRECTION_DOWN){
			newShoot = new Shoot(x-decalShootX,y-decalShootY-(cptDecal>decalStep?1:0),0,-shootSpeed,energyFrames[1][0]);
			if(shootMode>0){
				newShoot2 = new Shoot(x-decalShootX,y-decalShootY-(cptDecal>decalStep?1:0),-shootSpeed*0.66f,-shootSpeed*0.66f,energyFrames[1][0]);
				newShoot3 = new Shoot(x-decalShootX,y-decalShootY-(cptDecal>decalStep?1:0),shootSpeed*0.66f,-shootSpeed*0.66f,energyFrames[1][0]);
			}
			if(shootMode>1){
				newShoot4 = new Shoot(x-decalShootX,y-decalShootY-(cptDecal>decalStep?1:0),-shootSpeed*0.35f,-shootSpeed*0.9f,energyFrames[1][0]);
				newShoot5 = new Shoot(x-decalShootX,y-decalShootY-(cptDecal>decalStep?1:0),shootSpeed*0.35f,-shootSpeed*0.9f,energyFrames[1][0]);
			}
		}else if(direction == DIRECTION_UP){
			newShoot = new Shoot(x-decalShootX,y-decalShootY-(cptDecal>decalStep?1:0),0,shootSpeed,energyFrames[1][0]);
			if(shootMode>0){
				newShoot2 = new Shoot(x-decalShootX,y-decalShootY-(cptDecal>decalStep?1:0),-shootSpeed*0.66f,shootSpeed*0.66f,energyFrames[1][0]);
				newShoot3 = new Shoot(x-decalShootX,y-decalShootY-(cptDecal>decalStep?1:0),shootSpeed*0.66f,shootSpeed*0.66f,energyFrames[1][0]);
			}
			if(shootMode>1){
				newShoot4 = new Shoot(x-decalShootX,y-decalShootY-(cptDecal>decalStep?1:0),-shootSpeed*0.35f,shootSpeed*0.9f,energyFrames[1][0]);
				newShoot5 = new Shoot(x-decalShootX,y-decalShootY-(cptDecal>decalStep?1:0),shootSpeed*0.35f,shootSpeed*0.9f,energyFrames[1][0]);
			}
		}else if(direction == DIRECTION_LEFT){
			newShoot = new Shoot(x-decalShootX,y-decalShootY-(cptDecal>decalStep?1:0),-shootSpeed,0,energyFrames[1][0]);
			if(shootMode>0){
				newShoot2 = new Shoot(x-decalShootX,y-decalShootY-(cptDecal>decalStep?1:0),-shootSpeed*0.66f,-shootSpeed*0.66f,energyFrames[1][0]);
				newShoot3 = new Shoot(x-decalShootX,y-decalShootY-(cptDecal>decalStep?1:0),-shootSpeed*0.66f,shootSpeed*0.66f,energyFrames[1][0]);
			}
			if(shootMode>1){
				newShoot4 = new Shoot(x-decalShootX,y-decalShootY-(cptDecal>decalStep?1:0),-shootSpeed*0.9f,-shootSpeed*0.35f,energyFrames[1][0]);
				newShoot5 = new Shoot(x-decalShootX,y-decalShootY-(cptDecal>decalStep?1:0),-shootSpeed*0.9f,shootSpeed*0.35f,energyFrames[1][0]);
			}
		}else if(direction == DIRECTION_RIGHT){
			newShoot = new Shoot(x-decalShootX,y-decalShootY-(cptDecal>decalStep?1:0),shootSpeed,0,energyFrames[1][0]);
			if(shootMode>0){
				newShoot2 = new Shoot(x-decalShootX,y-decalShootY-(cptDecal>decalStep?1:0),shootSpeed*0.66f,-shootSpeed*0.66f,energyFrames[1][0]);
				newShoot3 = new Shoot(x-decalShootX,y-decalShootY-(cptDecal>decalStep?1:0),shootSpeed*0.66f,shootSpeed*0.66f,energyFrames[1][0]);
			}
			if(shootMode>1){
				newShoot4 = new Shoot(x-decalShootX,y-decalShootY-(cptDecal>decalStep?1:0),shootSpeed*0.9f,-shootSpeed*0.35f,energyFrames[1][0]);
				newShoot5 = new Shoot(x-decalShootX,y-decalShootY-(cptDecal>decalStep?1:0),shootSpeed*0.9f,shootSpeed*0.35f,energyFrames[1][0]);
			}
		}
		if(newShoot != null){
			shoots.add(newShoot);
			GdxUtils.SOUND_ENERGY_EXPLODE.play();
		}
		if(newShoot2 != null){
			shoots.add(newShoot2);
		}
		if(newShoot3 != null){
			shoots.add(newShoot3);
		}
		if(newShoot4 != null){
			shoots.add(newShoot4);
		}
		if(newShoot5 != null){
			shoots.add(newShoot5);
		}
	}
	
	public void randomShoot(OneRoom game){
		float shootX = (float) (Math.random()*shootSpeed);
		float shootY = shootSpeed - shootX;
		if(Math.random()<0.5){
			shootX*=-1;
		}
		if(Math.random()<0.5){
			shootY*=-1;
		}
		Shoot s =new Shoot(x-decalShootX,y-decalShootY-(cptDecal>decalStep?1:0),shootX,shootY,energyFrames[2][0]);
		shoots.add(s);
		GdxUtils.SOUND_ENERGY_DARK_SHOOT.play(0.6f);
	}

	
	
	public void touchByEnemy(OneRoom game,Personnage c){
		if(loadShield>0){
			if(cooldownTouch == 0){
				GdxUtils.SOUND_SHIELD_BLOCK.play();
				cooldownTouch = 8;
			}
		}else{
			if(cooldownTouch == 0){
				cooldownTouch = 25;
				this.life -= c.getAttackDmg();
				game.addAnimation(new SimpleAnimation(x-16,y-16,getRandomImpactPlayerAnimation()));
				if(life<=0){
					life=0;
					die(game);
				}
			}
		}
		
	}
	
	public boolean touchByShoot(OneRoom game,Shoot s){
		if(life>0 && Math.abs(s.getX()-x)<(cellWidth/2) && Math.abs(s.getY()-y)<(cellHeight/2)){
			if(loadShield>0){
				if(cooldownTouch == 0){
					GdxUtils.SOUND_SHIELD_BLOCK.play();
					cooldownTouch = 8;
				}
			}else{
				if(cooldownTouch == 0){
					cooldownTouch = 25;
					this.life -= 1;
					game.addAnimation(new SimpleAnimation(x-16,y-16,getRandomImpactPlayerAnimation()));
					if(life<=0){
						life=0;
						die(game);
					}
				}
			}
			return true;
		}
		return false;
		
	}
	
	public void die(OneRoom game){
		if(!dead){
		for(int i=0;i<20;i++){
			game.getAnimations().add(new SimpleAnimation((float)(x-32+Math.random()*64), (float)(y-32+Math.random()*64),getRandomImpactAnimation(),(int)(Math.random()*96)));
		}
		game.getTitle().initGameOver(game);
		dead=true;
		}
	}

	public void addXp(OneRoom game, int addXp){
		this.xp += addXp;
		if(levelLader[level+1]<xp){
			this.level++;
			game.getHud().addIncreasePoint();
			game.getAnimations().add(new SimpleAnimation(x-16, y, levelUpAnimation, 0));
			game.getHud().initMessage(game.getPlayer().getX()-64, game.getPlayer().getY(), "LEVEL UP !");
			GdxUtils.SOUND_LEVELUP.play();
		}
		
	}
	
	/**
	 * @return the energy
	 */
	public int getEnergy() {
		return energy;
	}

	/**
	 * @param energy the energy to set
	 */
	public void setEnergy(int energy) {
		this.energy = energy;
	}

	/**
	 * @return the maxEnergy
	 */
	public int getMaxEnergy() {
		return maxEnergy;
	}

	/**
	 * @param maxEnergy the maxEnergy to set
	 */
	public void setMaxEnergy(int maxEnergy) {
		this.maxEnergy = maxEnergy;
	}

	/**
	 * @return the level
	 */
	public int getLevel() {
		return level;
	}

	/**
	 * @return the xp
	 */
	public int getXp() {
		return xp;
	}

	/**
	 * @return the levelLader
	 */
	public int[] getLevelLader() {
		return levelLader;
	}
	
	public float getShootSpeed(){
		return shootSpeed;
	}
	
	public void setShootSpeed(float newShootSpeed){
		this.shootSpeed = newShootSpeed;
	}
	
	public void increaseAttack1(OneRoom game){
		energyLoad.setFrameDuration(0.04f);
		shootSpeed+=0.5f;
		System.out.println("SHOOT MODE="+shootMode);
	}
	
	public void increaseAttack2(OneRoom game){
		energyLoad.setFrameDuration(0.02f);
		shootSpeed+=0.2f;
		shootMode++;
		System.out.println("SHOOT MODE="+shootMode);
	}
	public void increaseAttack3(OneRoom game){
		energyLoad.setFrameDuration(0.01f);
		shootSpeed+=0.1f;
		shootMode++;
		System.out.println("SHOOT MODE="+shootMode);
	}
	
	public void increaseAttack4(OneRoom game){
		shootSpeed+=0.1f;
		shootMode++;
//		System.out.println("SHOOT MODE="+shootMode);
	}
	
	public void fasterInShield(float f){
		boostInShield = f;
	}

	public Animation[] getImpactAnimations(){
		return impactAnimations;
	}
	
	public Animation getRandomImpactAnimation(){
		return impactAnimations[(int)(Math.random()*4)];
	}
	public Animation getRandomImpactPlayerAnimation(){
		return impactPlayerAnimations[(int)(Math.random()*4)];
	}
	public Animation getHealAnimation(){
		return healAnimation;
	}
	public Animation getSpeedPotionAnimation(){
		return speedPotionAnimation;
	}
	
	public int getShootMode(){
		return shootMode;
	}
	
	public void setY(float newY){
		float dy = newY-y;
		for (Point point : blockpoint) {
			point.setY(point.getY()+dy);
		}
		this.y = newY;
	}
	
	public void addDarkPotion(OneRoom game){
		this.cptDarkPotion = 600;
	}
}
